@@ -154,7 +154,13 @@ def normalize_routine(routine):
             if obj[obj_name] is None:
                 obj[obj_name] = 'MINIMIZE'
 
-    # TODO: Normalize the constraints
+    # Normalize the constraints
+    try:
+        _ = config['constraints']
+        if not _:  # empty list
+            config['constraints'] = None
+    except KeyError:
+        config['constraints'] = None
 
     # Remove the additional info
     del routine['env_vranges']
@@ -236,9 +242,15 @@ def run_routine(routine, skip_review=False, save=None, verbose=2,
     # TODO: figure out the correct logic
     # It seems that the interface should be given rather than
     # initialized here
-    intf_name = configs_env['interface'][0]
-    Interface, _ = get_intf(intf_name)
-    intf = Interface()
+    try:
+        intf_name = configs_env['interface'][0]
+        Interface, _ = get_intf(intf_name)
+        intf = Interface()
+    except KeyError:
+        intf = None
+    except Exception as e:
+        logger.warn(e)
+        intf = None
 
     env = Environment(intf, routine['env_params'])
     if env_ready:
@@ -249,7 +261,7 @@ def run_routine(routine, skip_review=False, save=None, verbose=2,
     from .logger import _get_default_logger
     from .logger.event import Events
 
-    logger = _get_default_logger(verbose)  # log the optimization progress
+    opt_logger = _get_default_logger(verbose)  # log the optimization progress
     var_names = [next(iter(d)) for d in routine['config']['variables']]
     vranges = np.array([d[next(iter(d))]
                         for d in routine['config']['variables']])
@@ -258,12 +270,20 @@ def run_routine(routine, skip_review=False, save=None, verbose=2,
     pf = ParetoFront(rules)
     if pf_ready:
         pf_ready(pf)
+    if routine['config']['constraints']:
+        con_names = [next(iter(d)) for d in routine['config']['constraints']]
+        thresholds = [d[next(iter(d))] for d in routine['config']['constraints']]
+    else:
+        con_names = []
+        thresholds = []
 
     info = {'count': -1}
     # Make a normalized evaluate function
 
     def evaluate(X):
         Y = []
+        I = []
+        E = []
         for x in X:
             _x = denorm(x, vranges[:, 0], vranges[:, 1])
 
@@ -271,6 +291,8 @@ def run_routine(routine, skip_review=False, save=None, verbose=2,
                 before_evaluate(_x)
 
             env.set_vars(var_names, _x)
+
+            # Deal with objectives
             obses = []
             obses_raw = []
             for i, obj_name in enumerate(obj_names):
@@ -284,24 +306,53 @@ def run_routine(routine, skip_review=False, save=None, verbose=2,
             Y.append(obses)
             obses_raw = np.array(obses_raw)
 
+            # Deal with constraints
+            # TODO: Check overlapping between objs and cons
+            cons_i = []
+            cons_e = []
+            cons_raw = []
+            for i, con_name in enumerate(con_names):
+                relation, thres = thresholds[i]
+                con = float(env.get_obs(con_name))
+                if relation == 'GREATER_THAN':
+                    cons_i.append(con - thres)
+                elif relation == 'LESS_THAN':
+                    cons_i.append(thres - con)
+                else:
+                    cons_e.append(con - thres)
+                cons_raw.append(con)
+            if cons_i:
+                I.append(cons_i)
+            if cons_e:
+                E.append(cons_e)
+            cons_raw = np.array(cons_raw)
+
             info['count'] += 1
             _idx_x = np.insert(_x, 0, info['count'])  # keep the idx info
 
             is_optimal = not pf.is_dominated((_idx_x, obses_raw))
-            solution = (_x, obses_raw, is_optimal, var_names, obj_names)
-            logger.update(Events.OPTIMIZATION_STEP, solution)
+            solution = (_x, obses_raw, cons_raw, is_optimal, var_names, obj_names, con_names)
+            opt_logger.update(Events.OPTIMIZATION_STEP, solution)
 
             if after_evaluate:
-                after_evaluate(_x, obses_raw)
+                after_evaluate(_x, obses_raw, cons_raw)
 
         Y = np.array(Y)
+        if I:
+            I = np.array(I)
+        else:
+            I = None
+        if E:
+            E = np.array(E)
+        else:
+            E = None
 
-        return Y, None, None
+        return Y, I, E
 
     # Start the optimization
     print('')
-    solution = (None, None, None, var_names, obj_names)
-    logger.update(Events.OPTIMIZATION_START, solution)
+    solution = (None, None, None, None, var_names, obj_names, con_names)
+    opt_logger.update(Events.OPTIMIZATION_START, solution)
     try:
         if not callable(optimize):  # doing optimization through extensions
             configs = {
@@ -315,9 +366,9 @@ def run_routine(routine, skip_review=False, save=None, verbose=2,
 
         optimize(evaluate, configs)
     except Exception as e:
-        logger.update(Events.OPTIMIZATION_END, solution)
+        opt_logger.update(Events.OPTIMIZATION_END, solution)
         raise e
-    logger.update(Events.OPTIMIZATION_END, solution)
+    opt_logger.update(Events.OPTIMIZATION_END, solution)
 
 
 def ts_to_str(ts, format='lcls-log'):
